@@ -27,7 +27,6 @@
  */
 
 #include <stdlib.h>
-#include <stddef.h> /* for offsetof */
 #include <string.h>
 #include <assert.h>
 
@@ -39,6 +38,12 @@
 
 #define ARRAY_SIZE(x) (sizeof(x)/sizeof((x)[0]))
 
+struct mapi_stub {
+   const void *name;
+   int slot;
+   mapi_func addr;
+};
+
 /* define public_string_pool and public_stubs */
 #define MAPI_TMP_PUBLIC_STUBS
 #include "mapi_tmp.h"
@@ -50,7 +55,7 @@ static int next_dynamic_slot = MAPI_TABLE_NUM_STATIC;
 void
 stub_init_once(void)
 {
-#ifdef PTHREADS
+#ifdef HAVE_PTHREAD
    static pthread_once_t once = PTHREAD_ONCE_INIT;
    pthread_once(&once, entry_patch_public);
 #else
@@ -148,6 +153,28 @@ stub_find_dynamic(const char *name, int generate)
    return stub;
 }
 
+static const struct mapi_stub *
+search_table_by_slot(const struct mapi_stub *table, size_t num_entries,
+                     int slot)
+{
+   size_t i;
+   for (i = 0; i < num_entries; ++i) {
+      if (table[i].slot == slot)
+         return &table[i];
+   }
+   return NULL;
+}
+
+const struct mapi_stub *
+stub_find_by_slot(int slot)
+{
+   const struct mapi_stub *stub =
+      search_table_by_slot(public_stubs, ARRAY_SIZE(public_stubs), slot);
+   if (stub)
+      return stub;
+   return search_table_by_slot(dynamic_stubs, num_dynamic_stubs, slot);
+}
+
 void
 stub_fix_dynamic(struct mapi_stub *stub, const struct mapi_stub *alias)
 {
@@ -164,3 +191,39 @@ stub_fix_dynamic(struct mapi_stub *stub, const struct mapi_stub *alias)
    entry_patch(stub->addr, slot);
    stub->slot = slot;
 }
+
+/**
+ * Return the name of a stub.
+ */
+const char *
+stub_get_name(const struct mapi_stub *stub)
+{
+   const char *name;
+
+   if (stub >= public_stubs &&
+       stub < public_stubs + ARRAY_SIZE(public_stubs))
+      name = &public_string_pool[(unsigned long) stub->name];
+   else
+      name = (const char *) stub->name;
+
+   return name;
+}
+
+/**
+ * Return the slot of a stub.
+ */
+int
+stub_get_slot(const struct mapi_stub *stub)
+{
+   return stub->slot;
+}
+
+/**
+ * Return the address of a stub.
+ */
+mapi_func
+stub_get_addr(const struct mapi_stub *stub)
+{
+   assert(stub->addr || (unsigned int) stub->slot < MAPI_TABLE_NUM_STATIC);
+   return (stub->addr) ? stub->addr : entry_get_public(stub->slot);
+}
